import os
import sys

import pytest

from swg2pyt.GlobalDict import GlobalDict

from .Executor import DESCRIPTION, PRETREATMENT, WHI, Executor

py_template = '\
import pytest\n\
from swg2pyt.ClassGenerator import ClassGenerator\n\
\n\
methoddict, traceexe = ClassGenerator("{}","{}").getall()\n'
description_template = "\
'''\n\
    {}\n\
    '''"
mark_template = '\
@pytest.mark.{}\n'
method_template = '\n\n\
{}\
def test_{}{}:\n\
    {}\n\
    traceexe["{}"].exe(methoddict["{}"], "{}", {})\n'


class ClassGenerator(object):
    inst = None
    inited = False

    def __new__(cls, swg, casedir):
        if not cls.inst:
            cls.inst = super().__new__(cls)
        return cls.inst

    def __init__(self, swgdir, casedir) -> None:
        global py_template
        if not ClassGenerator.inited:
            self.swgs = []
            self.locate_swg(swgdir)
            ClassGenerator.inited = True
            self.classdict = {}
            self.methoddict = {}
            self.cmc = {}
            self.traceexe = {}
            self.fixturedict = {}
            self.descdict = {}
            self.casedir = casedir
            py_template = py_template.format(swgdir, casedir)
            # 遍历swagger文件夹，为每个api生成一个执行器Executor
            # 并更新self.xxxdict
            for swg in self.swgs:
                for v in swg.__dict__:
                    variable = swg.__getattribute__(v)
                    if type(variable).__name__ == "type":
                        variable = variable()
                        executor = Executor(variable, tuple([casedir]))
                        self.updatecmc(v, executor)
                        self.classdict[v] = variable

    def locate_swg(self, swgdir):
        cwd, dirs, files = next(os.walk(os.getcwd(), topdown=True))
        for dir in dirs:
            absdir = os.path.join(cwd, dir)
            if ".swagger-codegen-ignore" in os.listdir(absdir) and dir == swgdir:
                if absdir not in sys.path:
                    sys.path.append(absdir)
                swg = __import__("swagger_client.api", fromlist=[dir])
                if swg not in self.swgs:
                    self.swgs.append(swg)

    def updatecmc(self, classname, executor):
        casedict = executor.cc.casedict
        for m in executor.methods:
            mkey = m.__name__.replace(WHI, "")
            cmethod = self.cmc.setdefault(classname, {})
            mcase = cmethod.setdefault(mkey, [])
            self.methoddict[mkey] = m
            self.traceexe[mkey] = executor
            cdm = casedict.get(mkey.replace(WHI, ""))
            if cdm:
                for c in cdm:
                    mcase.append(c)
                    casetitle = f'{mkey}_{c.get("title")}'
                    self.descdict[casetitle] = c.get(DESCRIPTION)
        defaultdesc = {d: d for d in self.descdict if self.descdict[d] is None}
        self.descdict.update(defaultdesc)

    def getall(self):
        return self.methoddict, self.traceexe

    def generate(self):
        genpath = os.path.join(os.getcwd(), self.casedir, "auto")
        if not os.path.exists(genpath):
            os.mkdir(genpath)
        self.tofile(genpath)
        return genpath

    def generate_and_run(self, pytest_argv=[]):
        pytest.main([self.generate()] + pytest_argv)

    def tofile(self, path):
        for apiclass in self.cmc:
            filecontent = ""
            for cmethod in self.cmc[apiclass]:
                for mcase in self.cmc[apiclass][cmethod]:
                    fixture = self.genfixture(mcase.get(PRETREATMENT))
                    ctitle = f'{cmethod.replace(WHI,"")}_{mcase["title"]}'
                    dt = self.descdict[ctitle].replace("\n", "\n    ")
                    dt = description_template.format(dt)
                    strfixture1 = str(tuple(fixture)).replace("'", "")
                    strfixture2 = str({f'"{ft}"': ft for ft in fixture})
                    strfixture2 = strfixture2.replace("'", "")
                    mks = mcase.get("mark")
                    mkt = "".join(
                        [mark_template.format(mk)for mk in mks] if mks else "")
                    mtformat = (mkt, ctitle, strfixture1, dt, cmethod, cmethod,
                                mcase["title"], strfixture2)
                    mt = method_template.format(*mtformat)
                    filecontent += mt
            if filecontent:
                apipath = os.path.join(path, f"test_{apiclass}.py")
                with open(apipath, "w", encoding="utf-8")as test_newpy:
                    test_newpy.write(py_template+filecontent)
        with open(os.path.join(path, "marks"), "w+", encoding="utf-8")as markfile:
            marks = [mark for mark in GlobalDict()["marks"]]
            markfile.write(",".join(marks))

    def genfixture(self, ptm):
        rt = tuple()
        if ptm:
            fixture = ptm.get("fixture")
            if fixture:
                rt = fixture
        return rt
